

import torch
# import pandas as pd


class SoccerNetDataLoaderTensorFlow(torch.utils.data.Dataset):
    
    def __init__(self, soccernet_dir, jsonGamesFile=None, transform=None, input="Video_LQ"):
        """
        Args:
            csv_file (string): Path to the csv file with annotations.
            root_dir (string): Directory with all the images.
            transform (callable, optional): Optional transform to be applied
                on a sample.
        """

        if jsonGamesFile is None:
            jsonGamesFile = Path(__file__).parent / "data/SoccerNetGames.json"

        self.jsonGamesFile = jsonGamesFile

        with open(jsonGamesFile, "r") as json_file:
            dictionary = json.load(json_file)

        self.games = []
        for championship in dictionary:
            for season in dictionary[championship]:
                for game in dictionary[championship][season]:

                    self.games.append(os.path.join(championship, season, game))
                    

        # self.landmarks_frame = pd.read_csv(csv_file)
        self.soccernet_dir = soccernet_dir
        self.transform = transform
        self.input = input

    def __len__(self):
        return len(self.games)

    def __getitem__(self, idx):
        game = self.games[idx]

        if self.input == "Video_LQ":
            labels = scikit.io.vread(os.path.join(self.games[idx], "Labels.txt")
            video1 = scikit.io.vread(os.path.join(self.games[idx], "1.mkv")
            video2 = scikit.io.vread(os.path.join(self.games[idx], "2.mkv")

            sample={'video1': video1, 'video2': video2,
                    'labels': labels}

            if self.transform:
                sample = self.transform(sample)

        return sample




from .utils import getListTestGames, getListGames



class VideoLoader():
    def __init__(self, SoccerNetDir, split="v1"):
        self.SoccerNetDir = SoccerNetDir
        self.split = split
        if split == "v1":
            self.listGame = getListGames()
        elif split == "test":
            self.listGame = getListTestGames()
        else:
            self.listGame = getListGames(split)

    def __len__(self):
        return len(self.listGame)

    def __iter__(self, index):
        video_path = self.listGame[index]

        # Read RELIABLE lenght for the video, in second
        if args.verbose:
            print("video path", video_path)
        v = cv2.VideoCapture(video_path)
        v.set(cv2.CAP_PROP_POS_AVI_RATIO, 1)
        time_second = v.get(cv2.CAP_PROP_POS_MSEC)/1000
        if args.verbose:
            print("duration video", time_second)
        import json
        metadata = skvideo.io.ffprobe(video_path)
        # print(metadata.keys())
        # print(json.dumps(metadata["video"], indent=4))
        # getduration
        # print(metadata["video"]["@avg_frame_rate"])
        # # print(metadata["video"]["@duration"])

        # Knowing number of frames from FFMPEG metadata w/o without iterating over all frames
        videodata = skvideo.io.FFmpegReader(video_path)
        (numframe, _, _, _) = videodata.getShape()  # numFrame x H x W x channels
        if args.verbose:
            print("shape video", videodata.getShape())

        # # extract REAL FPS
        fps_video = metadata["video"]["@avg_frame_rate"]
        fps_video = float(fps_video.split("/")[0])/float(fps_video.split("/")[1])
        # fps_video = numframe/time_second
        if args.verbose:
            print("fps=", fps_video)
        time_second = numframe / fps_video
        if args.verbose:
            print("duration video", time_second)
        frames = []
        videodata = skvideo.io.vreader(video_path)
        fps_desired = 2
        drop_extra_frames = fps_video/fps_desired
        for i_frame, frame in tqdm(enumerate(videodata), total=numframe):
            # print(i_frame % drop_extra_frames)
            if (i_frame % drop_extra_frames < 1):

                if args.preprocess == "resize256crop224":  # crop keep the central square of the frame
                    frame = imutils.resize(frame, height=256)  # keep aspect ratio
                    # number of pixel to remove per side
                    off_side_h = int((frame.shape[0] - 224)/2)
                    off_side_w = int((frame.shape[1] - 224)/2)
                    frame = frame[off_side_h:-off_side_h,
                                off_side_w:-off_side_w, :]  # remove them

                elif args.preprocess == "crop":  # crop keep the central square of the frame
                    frame = imutils.resize(frame, height=224)  # keep aspect ratio
                    # number of pixel to remove per side
                    off_side = int((frame.shape[1] - 224)/2)
                    frame = frame[:, off_side:-off_side, :]  # remove them

                elif args.preprocess == "resize":  # resize change the aspect ratio
                    # lose aspect ratio
                    frame = cv2.resize(frame, (224, 224),
                                    interpolation=cv2.INTER_CUBIC)

                else:
                    raise NotImplmentedError()
                frames.append(frame)

        # create numpy aray (nb_frames x 224 x 224 x 3)
        frames = np.array(frames)
        return frames




import urllib.request
import os
from tqdm import tqdm
import json

from .utils import getListTestGames, getListGames

class MyProgressBar():
    def __init__(self, filename):
        self.pbar = None
        self.filename = filename

    def __call__(self, block_num, block_size, total_size):
        if not self.pbar:
            self.pbar = tqdm(total=total_size, unit='iB', unit_scale=True)
            self.pbar.set_description(f"Downloading {self.filename}...")
            self.pbar.refresh()  # to show immediately the update

        self.pbar.update(block_size)



class OwnCloudDownloader():
    def __init__(self, LocalDirectory, OwnCloudServer):
        self.LocalDirectory = LocalDirectory
        self.OwnCloudServer = OwnCloudServer

    def downloadFile(self, path_local, path_owncloud, user=None, password=None):
        # return 0: successfully downloaded
        # return 1: HTTPError
        # return 2: unsupported error
        # return 3: file already exist locally

        if user is not None or password is not None:  
            # update Password
             
            password_mgr = urllib.request.HTTPPasswordMgrWithDefaultRealm()
            password_mgr.add_password(
                None, self.OwnCloudServer, user, password)
            handler = urllib.request.HTTPBasicAuthHandler(
                password_mgr)
            opener = urllib.request.build_opener(handler)
            urllib.request.install_opener(opener)

        if os.path.exists(path_local): # check existence
            print(f"{path_local} already exists")
            return 2

        try:
            try:
                urllib.request.urlretrieve(
                    path_owncloud, path_local, MyProgressBar(path_local))

            except urllib.error.HTTPError as identifier:
                print(identifier)
                return 1
        except:
            os.remove(path_local)
            raise
            return 2
        return 0


class SoccerNetDownloader(OwnCloudDownloader):
    def __init__(self, LocalDirectory,
                 OwnCloudServer="https://exrcsdrive.kaust.edu.sa/exrcsdrive/public.php/webdav/"):
        super(SoccerNetDownloader, self).__init__(
            LocalDirectory, OwnCloudServer)

    def downloadGames(self, files=["1.mkv", "2.mkv", "Labels.json"]):

        for game in getListGames():

            # game = os.path.join(championship, season, game)


            for file in files:

                GameDirectory = os.path.join(self.LocalDirectory, game)
                FileURL = os.path.join(
                    self.OwnCloudServer, game, file).replace(' ', '%20')
                os.makedirs(GameDirectory, exist_ok=True)

                # LQ Videos
                if file in ["1.mkv", "2.mkv"]:
                    res = self.downloadFile(path_local=os.path.join(GameDirectory, file),
                                            path_owncloud=FileURL,
                                            user="6XYClm33IyBkTgl",  # user for video LQ
                                            password=self.password)

                # HQ Videos
                elif file in ["1_HQ.mkv", "2_HQ.mkv", "video.ini"]:
                    res = self.downloadFile(path_local=os.path.join(GameDirectory, file),
                                            path_owncloud=FileURL,
                                            user="B72R7dTu1tZtIst",  # user for video HQ
                                            password=self.password)

                # Labels
                elif file in ["Labels.json"]:
                    res = self.downloadFile(path_local=os.path.join(GameDirectory, file),
                                            path_owncloud=FileURL,
                                            user="ZDeEfBzCzseRCLA",  # user for Labels
                                            password="SoccerNet")
                
                # features
                elif any(feat in file for feat in ["ResNET", "C3D", "I3D"]):
                    res = self.downloadFile(path_local=os.path.join(GameDirectory, file),
                                            path_owncloud=FileURL,
                                            user="9eRjic29XTk0gS9",  # user for Features
                                            password="SoccerNet")

    def downloadTestGames(self, files=["1.mkv", "2.mkv", "Labels.json"]):

        for game in getListTestGames():

            # game = os.path.join(championship, season, game)


            for file in files:

                GameDirectory = os.path.join(self.LocalDirectory, game)
                FileURL = os.path.join(
                    self.OwnCloudServer, game, file).replace(' ', '%20')
                os.makedirs(GameDirectory, exist_ok=True)

                # LQ Videos
                if file in ["1.mkv", "2.mkv"]:
                    res = self.downloadFile(path_local=os.path.join(GameDirectory, file),
                                            path_owncloud=FileURL,
                                            user="trXNXsW9W04onBh",  # user for video LQ
                                            password=self.password)

                # HQ Videos
                elif file in ["1_HQ.mkv", "2_HQ.mkv", "video.ini"]:
                    res = self.downloadFile(path_local=os.path.join(GameDirectory, file),
                                            path_owncloud=FileURL,
                                            user="gJ8gja7V8SLxYBh",  # user for video HQ
                                            password=self.password)

                # Labels
                elif file in ["Labels.json"]:
                    res = self.downloadFile(path_local=os.path.join(GameDirectory, file),
                                            path_owncloud=FileURL,
                                            user="WUOSnPSYRC1RY13",  # user for Labels
                                            password=self.password)

                # Labels
                elif any(feat in file for feat in ["ResNET", "C3D", "I3D"]):
                    res = self.downloadFile(path_local=os.path.join(GameDirectory, file),
                                            path_owncloud=FileURL,
                                            user="d4nu5rJ6IilF9B0",  # user for Features
                                            password="SoccerNet")


                    
